@@ -39,6 +39,7 @@ import urllib.request
 import os
 import collections
 import pathlib
+import random
 import sys
 import re
 import json
@@ -1574,6 +1575,88 @@ def schedules_timeline(context, team_name, history, expand, interval, intervalun
     print(format_table)
 
 
+@schedules.command(name='reshuffle')
+@click.option('--team-name', 'team_name', required=True)
+@click.option('--start-date', 'start_date', help='Example: 2020-03-20', required=True)
+@click.option('--end-date', 'end_date', help='Example: 2020-06-27', required=True)
+@click.option('--exclude-engineer', 'exclude_engineer')
+@click.pass_context
+def schedules_timeline(context, team_name, start_date, end_date, exclude_engineer):  # pylint: disable=too-many-locals, too-many-arguments
+    """
+    Function to list the schedules for all teams
+
+    Returns:
+        Table output listing the schedules for all teams in the format of two columns: id and name
+
+    """
+    ## get current schedule
+    team_schedule_name = find_team_schedule_name_by_name(team_name)  # pylint: disable=no-value-for-parameter
+    result = context.obj['opsgenie'].list_schedule_timeline_by_team_name(team_schedule_name, 'base', 2, 'months')
+
+    ### get all engineers that will be on call
+    unique_engineers = list(set([x['recipient']['name'] for x in result['data']['finalTimeline']['rotations'][0]['periods']]))
+    if exclude_engineer:
+        engineer_to_remove = [x for x in unique_engineers if re.findall(rf"\b{exclude_engineer}\b", x, re.IGNORECASE)]
+        unique_engineers.remove(engineer_to_remove[0])
+
+    #### STILL TO DO: what if more than 1 engineer to exclude?
+
+    ## show old schedule, filtered down in time:
+    format_table = PrettyTable(['user', 'startdate', 'enddate'])
+    print(f'\n Current On-call schedule')
+    start_date = datetime.strptime(start_date, "%Y-%m-%d").date()
+    end_date = datetime.strptime(end_date, "%Y-%m-%d").date()
+    all_start_dates = []
+    for item in result['data'][f'finalTimeline']['rotations'][0]['periods']:
+        item_date_value = datetime.strptime((item['startDate'].replace('Z', '')).split(".")[0], "%Y-%m-%dT%H:%M:%S").date()
+        if start_date <= item_date_value and item_date_value < end_date:
+            all_start_dates += [item['startDate'].split("Z")[0]]
+            time_zone = pytz.timezone("UTC")
+
+            start_dt_truncated = (item['startDate'].replace('Z', '')).split(".")[0]
+            start_utc_dt = time_zone.localize((datetime.fromisoformat(start_dt_truncated)))
+            start_ams_dt = (start_utc_dt.astimezone(pytz.timezone("Europe/Amsterdam"))).strftime('%Y-%m-%d %H:%M:%S')
+
+            end_dt_truncated = (item['endDate'].replace('Z', '')).split(".")[0]
+            end_utc_dt = time_zone.localize((datetime.fromisoformat(end_dt_truncated)))
+            end_ams_dt = (end_utc_dt.astimezone(pytz.timezone("Europe/Amsterdam"))).strftime('%Y-%m-%d %H:%M:%S')
+
+            format_table.add_row([item['recipient']['name'], start_ams_dt, end_ams_dt])
+    print(format_table)
+
+    ## get all the days for that period
+    all_start_dates
+    format_table = PrettyTable(['user', 'startdate', 'enddate'])
+    print(f'\n Proposed On-call schedule')
+    copy_list_unique_engineers = list(unique_engineers)
+    last_engineer = ""
+    for date in all_start_dates:
+        time_zone = pytz.timezone("UTC")
+
+        while True:
+            randomNumber = random.randint(0, (len(copy_list_unique_engineers) - 1))
+            engineer = copy_list_unique_engineers[randomNumber]
+            if engineer != last_engineer:
+                break
+
+        end_date_x = str((datetime.strptime(date, "%Y-%m-%dT%H:%M:%S").date() + timedelta(days=1)))
+
+        start_utc_dt = time_zone.localize((datetime.fromisoformat(date)))
+        start_ams_dt = (start_utc_dt.astimezone(pytz.timezone("Europe/Amsterdam"))).strftime('%Y-%m-%d %H:%M:%S')
+
+        stop_utc_dt = time_zone.localize((datetime.fromisoformat(end_date_x)))
+        stop_ams_dt = (start_utc_dt.astimezone(pytz.timezone("Europe/Amsterdam"))).strftime('%Y-%m-%d %H:%M:%S')
+
+        format_table.add_row([engineer, start_ams_dt, stop_ams_dt])
+        last_engineer = format_table._rows[-1][0]
+        del copy_list_unique_engineers[randomNumber]
+        if not copy_list_unique_engineers:
+            copy_list_unique_engineers = list(unique_engineers)
+    print(format_table)
+
+    answer = input("\nDo you want to override the old schedule with the proposed schedule? [y/n]: ").lower().strip()
+
+
 @schedules.command(name='list')
 @click.pass_context
 def schedules_list(context):
